<?php
class Ci_sms {
    private $CI;
    private $sms_client;
    private $sms_type = 0;
    public function __construct() {
        $this->CI = & get_instance();
        $this->CI->load->model('setting_model');
        $this->CI->load->model('template_model');
        $sms_type = get_config_item('sms_type','sms');
        if($this->sms_client==null){
            $this->load_sms_client($sms_type);
        }
    }

    /**
     * 加载短信接口类
     * @param $sms_type
     */
    public function load_sms_client($sms_type){
        $this->sms_type = $sms_type;
        switch ($sms_type){
            case 1:
                $sms_key = 'aliyunsms';
                break;
            case 2:
                $sms_key = 'baosms';
                break;
            default:
                $sms_key = 'tencentsms';
        }
        
        if($sms_type!=4 && has_addons($sms_key)){
            $sms_name = str_replace('sms','',$sms_key);
            $this->CI->load->add_addon_package($sms_key)->library('sms_'.$sms_name);
            $this->sms_client = $this->CI->{'sms_'.$sms_name};
        }else if($sms_type==100){
            // 短信屋
            $this->CI->load->library('sms/sms_quan');
            $this->sms_client = $this->CI->sms_quan;
        }else{
            // 比利联
            $this->CI->load->library('sms/sms_bll');
            $this->sms_client = $this->CI->sms_bll;
        }
    }

    // 判断是否支持某个功能和模块
    public function check_disable($module){
        $disable = $this->get_disable();
        if(in_array($module,$disable)){ return false; }
        return true;
    }

    // 获取不支持的模块
    public function get_disable(){
        $disable = [];
        if(property_exists($this->sms_client,'disable')){
            $disable = $this->sms_client->disable;
        }
        return $disable;
    }

    /**
     * 短信接口是否开启
     * @param string $template_mark 判断模板是否开启
     * @return bool
     */
    public function is_power($template_mark=''){
        $power = false;
        // 短信接口总开关
        $smsPower = get_config_item('sms_power','sms');
        // 第三方接口是否已配置启用
        $isSupport = $this->sms_client->isSupport();
        if($smsPower && $isSupport){
            $power = true;
            if($template_mark){
                // 检查短信模板是否已开启
                $template = $this->CI->template_model->find($template_mark);
                if(! element('status_sms', $template)){
                    $power = false;
                }
            }
        }
        return $power;
    }

    /**
     * 发送验证码短信（系统模板）
     * @param $mobile 接受者
     * @param $mark 短信模板标签 setting_model->sms_template
     * @param array $vars 变量值
     * @param array $input
     */
    public function send_template($mobile, $mark, $vars=[], $input=[]){
        try {
            // 判断是否启用
            if(! $this->is_power($mark)){
                return response(['code'=>1,'msg'=>_('短信功能或模板未启用')]);
            }

            // 验证手机号码
            $mobile = $original_mobile =trim($mobile);
            if(empty($mobile)){
                return response(['code'=>1,'msg'=>_('手机号码不能为空')]);
            }

            // 检查手机号格式
            $this->CI->load->model("country_model");
            $country_id = element('country_id',$input,$this->CI->country_model->default);
            $country_id = $country_id ?: $this->CI->country_model->default;
            $country = $this->CI->country_model->find(['id'=>$country_id]);
            if(empty($country)){
                return response(['code'=>1,'msg'=>_('请选择正确的国家')]);
            }
            if(!empty($country['preg'])){
                if(!preg_match($country['preg'],$mobile)){
                    //return response(['code'=>1,'msg'=>_('手机号码格式不正确')]);
                }
            }

            // 验证码模板
            $verify_marks = ['register_verify','login_verify','forget_verify','bind_verify','check_verfiy'];

            // 是否为异步日志（需要主动查询发送状态）
            $is_sync = 0;

            // 极验证插件
            if(in_array($mark,['register_verify','login_verify','forget_verify','bind_verify'])){
                if(has_addons("geetest")){
                    // 二次验证
                    try {
                        $this->CI->hooks->call_hook('geetest_validate',$input);
                    }catch (Exception $e){
                        return response(['code'=>201,'msg'=>$e->getMessage()]);
                    }
                }elseif(array_key_exists('captcha',$input)){
                    // 检查图形验证码
                    $captcha = element('captcha',$input);
                    if(!check_captcha($captcha)){
                        return response(['code'=>1,'msg'=>_('请输入正确的图形验证码')]);
                    }
                }
            }

            $uid = element('uid',$input,0);
            if(in_array($mark,['bind_verify'])){
                // 需要登录的模板
                if(! $uid){
                    return response(['code'=>1,'msg'=>_('请先登录')]);
                }
            }elseif(in_array($mark, ['login_verify','forget_verify'])){
                // 需要判断手机号是否注册的模板
                if(! $this->CI->user_model->find(['user_type'=>1,'mobile'=>$mobile],'id')){
                    return response(['code'=>1,'msg'=>_('该手机号未注册')]);
                }
            }

            // 获取短信模板CODE
            $sms_sign = $this->CI->config->item('sms_quan_sid','sms');
            $template = $this->CI->template_model->find($mark);
            $ali_code = element('code_sms', $template, '');
            if($this->check_disable('template_code')){
                if(! $ali_code){
                    //return response(['code'=>1,'msg'=>_('短信模板不存在')]);
                }
            }

            // 国际号码增加代码
            if($country_id!=$this->CI->country_model->default){
                $mobile = "+".$country['code'].$mobile;
            }

            // 验证码短信，发送间隔（一分钟）
            // 登陆后，同一个会员同一个模板
            // 未登录，同一个手机用一个模板
            $is_verify_sms = false;
            if(in_array($mark, $verify_marks)){
                $is_verify_sms = true;
                $where = ['template_mark'=>$mark];
                if($uid){ $where['uid'] = $uid; }
                else{ $where['to'] = $mobile; }
                $log = $this->CI->template_model->find_log($where);
                if(is_array($log)){
                    $time_add = element('time_add', $log);
                    if(now() - $time_add < 60){
                        return response(['code'=>1,'msg'=>_('发送太频繁（间隔一分钟）')]);
                    }
                }
            }else{
                // 官方接口/非验证码模板 发送状态为异步查询
                if($this->sms_type == 4){
                    $is_sync = 1;
                }
            }
            
            // 获取模板内容
            $template_sms = element('template_sms',$template,'');
            
            // 处理验证码变量
            $code = '';
            if(isset($vars['code']) && ! element('code', $vars)){
                $code = random_string('numeric',6);
                if(stripos($template_sms,'{$verify_code}') !== false){
                    $vars['verify_code'] = $code;
                    unset($vars['code']);
                }else{
                    $vars['code'] = $code;
                    unset($vars['verify_code']);
                }
            }

            // 获取短信内容（替换变量）
            $template_txt = element('template_sms',$template,'');
            $template_txt = str_replace('${','{$',$template_txt);
            $template_sms = str_replace_vars($template_sms,$vars);//$this->CI->view('string:'.$template_sms,$vars,true);

            // 排除不支持的变量
            /*$vars_no_ava_sms = $this->CI->template_model->vars_no_ava_config('sms');
            $no_vars_sms = element($mark, $vars_no_ava_sms, []);
            if($vars){
                foreach($vars as $k=>$v){
                    if(in_array($k, $no_vars_sms)){
                        unset($vars[$k]);
                    }
                }
            }*/

            // 营销短信去htm;
            if($mark == 'push_template'){
                $template_sms = strip_tags($template_sms);
            }

            // 写发送日志
            $log_id = $this->CI->template_model->save_log([
                'template_type' => 1,
                'template_id' => $template['id'],
                'to' => $mobile,
                'uid' => $uid,
                'status' => 0,
                'content' => $template_sms,
                'api_type' => $this->sms_type,
                'sign_name' => $sms_sign,
                'is_sync' => $is_sync
            ]);
            if(empty($log_id)){
                return response(['code'=>1,'msg'=>_('短信发送异常')]);
            }

            if($is_sync){
                // 添加异步短信后，更新发送异步短信最后时间
                $cache_key = 'send_sms_last_sync_time';
                $this->CI->cache->save($cache_key, now(), false, 'redis');
            }

            // 传入第三方接口发送
            $response = $this->sms_client->send([
                'mobile' => $mobile, // 接收手机号
                'template_code' => $ali_code, // 第三方模板标识
                'template_sms' => $template_sms, // 发送模板内容（已替换变量）
                'template_txt' => $template_txt, // 模板原文
                'sign_name' => $sms_sign, // 签名
                'vars' => $vars, // 模板变量
                'log_id' => $log_id, // 模板日志ID
                'is_sync' => $is_sync
            ]);
            $api_request = element('api_request', $response, '');
            $api_response = element('api_response', $response, '');
            unset($response['api_request'],$response['api_response']);

            $post = [
                'api_request' => $api_request,
                'api_response' => $api_response,
            ];

            $response_code = element('code',$response);
            if($response_code > 0){ // 发送失败
                $post['status_msg'] = element('msg',$response,'');
            }else{ // 发送成功
                $post['status'] = 1;
            }
            $this->CI->template_model->save_log($post,['id'=>$log_id]);
            if(element('code',$response) > 0){
                $response['msg'] = $is_verify_sms ? _('短信验证码发送失败') : _('短信发送失败');
                return $response;
            }

            // 生成验证码变量session（一小时内过期）
            $msg = _("短信已发送！");
            if($code){
                $this->CI->session->set_tempdata('verify_mobile', $original_mobile, 60*5);
                $this->CI->session->set_tempdata('verify_code', $code, 60*5);
                $msg .= '（'._('5分钟内有效').'）';
            }

            return response($msg);
        } catch (ClientException $e) {
            $msg = $e->getErrorMessage();
            return response(['code'=>1,'msg'=>$msg]);
        } catch (ServerException $e) {
            $msg = $e->getErrorMessage();
            return response(['code'=>1,'msg'=>$msg]);
        }
    }

    /**
     * 查询短信模板状态
     * @param $template_code
     */
    public function query_sms_template($mark, $template_code){
        if(! $this->is_power($mark)){
            return response(['code'=>1,'msg'=>_('短信功能或模板未启用')]);
        }
        $response = $this->sms_client->querySmsTemplate($template_code);
        $template_status = element('template_status',$response);
        if(element('code',$response) == 0){
            if(in_array($template_status,[0,1])){ // 审核中/审核成功
                $response['template_status_info'] = $this->template_status_config(element('template_status',$response));
                $response['template_type_info'] = $this->template_type_config(element('template_type',$response));
            }else{ // 审核失败/其他状态
                $response['code'] = 1;
                if($template_status == 10){ // 已撤回
                    $response['msg'] = _('审核已撤回');
                }else{
                    $response['msg'] = _('审核失败').'：'.element('reason',$response);
                }
            }
        }
        return $response;
    }

    /**
     * 申请短信模板
     * @param $input
     */
    public function apply_sms_template($mark, $input){
        if(! $this->is_power($mark)){
            return response(['code'=>1,'msg'=>_('短信功能或模板未启用')]);
        }
        $response = $this->sms_client->applySmsTemplate($input);
        if(element('code',$response) == 0){
            $response['template_status_info'] = $this->template_status_config(element('template_status',$response));
        }
        return $response;
    }

    /**
     * 修改短信模板
     * @param $input
     */
    public function update_sms_template($mark, $input){
        if(! $this->is_power($mark)){
            return response(['code'=>1,'msg'=>_('短信功能或模板未启用')]);
        }
        $response = $this->sms_client->updateSmsTemplate($input);
        if(element('code',$response) == 0){
            $response['template_status_info'] = $this->template_status_config(element('template_status',$response));
        }
        return $response;
    }

    /**
     * 查询短信接口信息
     */
    public function query_sms_info($sms_type=''){
        if($sms_type !== ''){
            $this->load_sms_client($sms_type);
        }
        return $this->sms_client->query_client_info();
    }

    // 配置模板审核状态
    public function template_status_config($key = '', $field = '', $type = ''){
        $config = [
            '0' =>  ['name' => _('审核中'),'color'=>'warning'],
            '1' => ['name' => _('审核通过'),'color'=>'success'],
        ];
        return $this->CI->config->get_self_config($config,$key,$field,$type);
    }

    // 配置短信模板类型
    public function template_type_config($key = '', $field = '', $type = ''){
        $config = [
            '0' =>  ['name' => _('验证码')],
            '1' => ['name' => _('短信通知')],
            '2' => ['name' => _('推广短信')],
            '3' => ['name' => _('国际/港澳台消息')],
        ];
        return $this->CI->config->get_self_config($config,$key,$field,$type);
    }

}
